import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:layout_form_builder/layout_form_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';
import 'datalayer/bean/eform_page_json_form_data.dart';
import 'datalayer/bean/eform_source.dart';

class FormAndLayoutBuild extends StatefulWidget {
  final FormSource source;
  final Function genInitFormData;
  final String pageId;

  FormAndLayoutBuild({Key? key, required this.source, required this.genInitFormData, required this.pageId}) : super(key: key);

  @override
  _FormAndLayoutBuildState createState() => _FormAndLayoutBuildState();
}

class _FormAndLayoutBuildState extends State<FormAndLayoutBuild> {
  late Future<PageJsonFormData> pageJsonFormData;

  @override
  void initState() {
    super.initState();
    _loadPageUI();
  }

  void _loadPageUI() {
    pageJsonFormData = PageJsonFormData.from(widget.source, widget.genInitFormData, widget.pageId);
  }

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<PageJsonFormData>(
      future: pageJsonFormData,
      builder: (context, snapshot) {
        if (snapshot.hasData) {
          return LayoutForm(
            clickListener: DefaultClickListener(),
            pageJsonFormData: snapshot.data!
          );
        } else if (snapshot.hasError) {
          return Text('Error: ${snapshot.error}', style: TextStyle(color: AppColors.TextGray, fontSize: 14));
        }
        return SizedBox(
          width: 30,
          height: 30,
          child: CircularProgressIndicator()
        );
      });
  }
}

class DefaultClickListener implements ClickListener {
  @override
  void onClicked(String? event) {}
}
