import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/datalayer/constant/eform_properties.dart';
import 'package:layout_form_builder/eform_layout_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';
import '../util.dart';

class CheckboxGroup extends StatefulWidget {
  static const PROP = [
    EFormProperties.PROP_INPUT_ID,
    EFormProperties.PROP_COMPULSORY,
    EFormProperties.PROP_ON_LOAD_TRIGGERS,
    EFormProperties.PROP_ON_END_EDITING_TRIGGERS,
    EFormProperties.PROP_COMPULSORY_ERR_MSG,
    EFormProperties.PROP_INPUT_LABEL,
    EFormProperties.PROP_INPUT_ENABLED,
    EFormProperties.PROP_COLLABORATION_GROUP_ID,
    EFormProperties.PROP_DEFAULT_VALUE,
    EFormProperties.PROP_OPTIONS,
    EFormProperties.PROP_ALLOW_MULTI_SELECT,
    EFormProperties.PROP_ALLOW_INPUT_OTHER,
    EFormProperties.PROP_INPUT_VISIBLE,
    EFormProperties.PROP_VISIBLE_ON_PDF,
  ];

  final Map<String, dynamic> fieldProp;
  final ClickListener? listener;

  CheckboxGroup({Key? key, required this.fieldProp, required this.listener}) : super(key: key);

  @override
  _CheckboxGroupState createState() => _CheckboxGroupState();
}

class _CheckboxGroupState extends State<CheckboxGroup> {
  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    final hint = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_HINT, fromValidator: false);
    final optStr = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_OPTIONS, fromValidator: false);
    final inputId = EFormCache.fieldString(fp, EFormProperties.PROP_INPUT_ID);
    bool allowMultiSelect = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_ALLOW_MULTI_SELECT, defaultValue: true);

    final textStyle = EFormCache.getTextStyle();
    final inputDecoration = EFormCache.getInputDecoration();

    return Theme(
      data: ThemeData(
        primaryColor: HexColor.fromHex("#9599B6"),
        primaryColorDark: HexColor.fromHex("#9599B6"),
      ),
      child: FormBuilderCheckboxGroup(
        validator: (val) {
          return _validate(val, context);
        },
        checkColor: AppColors.AccentBright,
        activeColor: AppColors.TextGray,
        decoration: inputDecoration,
        name: inputId ?? "",
        options: (optStr == null ? '' : optStr)
            .split("@@@")
            .map((optItem) => FormBuilderFieldOption(value: optItem, child: Text(optItem, style: textStyle?.textStyle)))
            .toList(),
        onChanged: (val) {},
        separator: const VerticalDivider(
          width: 10,
          thickness: 5,
          color: Colors.transparent,
        ),
      )
    );
  }

  String? _validate(List<dynamic>? value, BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    bool compulsory = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_COMPULSORY, fromValidator: true, defaultValue: false);

    if (compulsory) {
      if (value == null || value.length == 0) {
        return EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_COMPULSORY_ERR_MSG, fromValidator: true);
      }
    }
    return null;
  }
}
