import 'package:dynamic_widget/dynamic_widget.dart';
import 'package:flutter/material.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/datalayer/constant/eform_properties.dart';
import 'package:layout_form_builder/eform_layout_renderer.dart';
import 'package:layout_form_builder/ui/app_colors.dart';

class DropDown extends StatefulWidget {
  static const PROP = [
    EFormProperties.PROP_INPUT_ID,
    EFormProperties.PROP_COMPULSORY,
    EFormProperties.PROP_ON_LOAD_TRIGGERS,
    EFormProperties.PROP_ON_END_EDITING_TRIGGERS,
    EFormProperties.PROP_COMPULSORY_ERR_MSG,
    EFormProperties.PROP_INPUT_LABEL,
    EFormProperties.PROP_INPUT_HINT,
    EFormProperties.PROP_INPUT_ENABLED,
    EFormProperties.PROP_COLLABORATION_GROUP_ID,
    EFormProperties.PROP_DEFAULT_VALUE,
    EFormProperties.PROP_OPTIONS,
    EFormProperties.PROP_ALLOW_MULTI_SELECT,
    EFormProperties.PROP_INPUT_VISIBLE,
    EFormProperties.PROP_VISIBLE_ON_PDF,
  ];

  final Map<String, dynamic> fieldProp;
  final ClickListener? listener;

  DropDown({Key? key, required this.fieldProp, required this.listener}) : super(key: key);

  @override
  _DropDownState createState() => _DropDownState();
}

class _DropDownState extends State<DropDown> {
  bool _gotData = false;
  String _prevInputStr = "";
  late FocusNode txtFocusNode;

  @override
  void initState() {
    _prevInputStr = EFormCache.getFormFieldInitValue(widget.fieldProp) ?? "";
    _gotData = _prevInputStr != "";
    super.initState();
    txtFocusNode = FocusNode();
    txtFocusNode.addListener(() {
      if (txtFocusNode.hasFocus) {
        print("[DropDown] Gain focus");
      } else {
        print("[DropDown] Lost focus");
      }
    });
  }

  @override
  void dispose() {
    // Clean up the focus node when the Form is disposed.
    txtFocusNode.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    Map<String, dynamic> fp = widget.fieldProp;
    int? maxLength = EFormCache.getFormFieldPropInt(context, fp, EFormProperties.PROP_MAX_LENGTH, fromValidator: true);
    maxLength = maxLength == 0 ? null : maxLength;
    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);
    String? label = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_LABEL);
    String? hint = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_INPUT_HINT);
    String? options = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_OPTIONS) ?? "";

    final textStyle = EFormCache.getTextStyle();
    final inputDecoration = EFormCache.getInputDecoration(label: label ?? "", hint: hint ?? "");

    return Theme(
        data: new ThemeData(
          primaryColor: AppColors.TextGray,
          primaryColorDark: AppColors.TextGray,
        ),
        child: Padding(
          padding: EdgeInsets.only(bottom: 10),
          child: FormBuilderDropdown<String>(
              items: options.split("@@@").map((e){
                return DropdownMenuItem(
                  child: Text(e, style: textStyle?.textStyle ?? TextStyle(fontSize: 14, color: AppColors.TextBlack, decoration: TextDecoration.none)),
                  value: e,
                );
              }).toList(),
              style: textStyle?.textStyle ?? TextStyle(fontSize: 14.0, fontFamily: 'Lato', color: Colors.black),
              name: fp[EFormProperties.PROP_INPUT_ID],
              enabled: enabled,
              autovalidateMode: AutovalidateMode.onUserInteraction,
              decoration: inputDecoration,
              isDense: true,
              validator: (val) {
                return _validate(val);
              },
              onChanged: (str) {
                String editedTxt = str;
                bool validInput = true;

                if (validInput) {
                  _prevInputStr = editedTxt;
                  setState(() {
                    _gotData = (editedTxt).length > 0;
                  });
                }
                else{
                  EFormCache.setFormFieldValue(fp, _prevInputStr);
                }
              },
              focusNode: txtFocusNode
          ),
        )
    );
  }

  String? _validate(dynamic value) {
    Map<String, dynamic> fp = widget.fieldProp;
    bool compulsory = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_COMPULSORY, fromValidator: true, defaultValue: false);
    String compulsoryErrMsg = EFormCache.getFormFieldPropString(context, fp, EFormProperties.PROP_COMPULSORY_ERR_MSG, fromValidator: true) ?? "";

    bool enabled = EFormCache.getFormFieldPropBool(context, fp, EFormProperties.PROP_INPUT_ENABLED, defaultValue: true);

    if (enabled) {
      String? val = value;
      if (compulsory) {
        if (val == null || val.length == 0) {
          return compulsoryErrMsg;
        }
      }
    }
    return null;
  }

}
