import 'dart:convert';

import 'package:flutter/material.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/formcomponent/full_size_column.dart';
import 'package:layout_form_builder/formcomponent/txt_field.dart';
import 'package:layout_form_builder/page_route_mgmt.dart';
import 'package:provider/provider.dart';

import 'app_setting.dart';
import 'datalayer/bean/eform_source.dart';
import 'form_and_layout_build.dart';
import 'util.dart';
import 'layout_form_renderer.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:http/http.dart' as http;
import 'package:animate_do/animate_do.dart';

import 'widget_prop_change_notifier.dart';

class HospitalisationPage extends StatefulWidget {
  @override
  _HospitalisationPageState createState() => _HospitalisationPageState();
}

class _HospitalisationPageState extends State<HospitalisationPage> {
  String _pageId = "";
  late FormSource _source;
  Key clientFormKey = GlobalKey<FormBuilderState>();
  bool _showDirectCredit = false;
  late PageRouteMgmt routeMgmt;
  AnimationController? animationCtrl;

  @override
  void initState() {
    _pageId = "hospitalisation";
    _source = FormSource(url: "${AppSetting.serverUrlPrefix}/flutterJson/#pageId#");
    super.initState();
  }

  Future<void> _saveBankingFormData() async {
    Map<String, dynamic>? values = EFormCache.getFormValues(_pageId);
    print("value is $values");
    if (values != null) {
      await http.post(
          Uri.parse("${AppSetting.serverUrlPrefix}/saveFlutterJson/$_pageId"),
          headers: <String, String>{
            'Content-Type': 'application/json; charset=UTF-8',
          },
          body: jsonEncode(values));
      routeMgmt.push(PageType.PAGE_FORM_GEN_PDF, _pageId);
    }
  }

  @override
  Widget build(BuildContext context) {
    routeMgmt = PageRouteMgmt(context);

    return Scaffold(
        backgroundColor: Colors.grey[300],
        appBar: AppBar(
          title: Text('Claim - Hospitalistion Form 1',
              style: TextStyle(fontFamily: 'Lato')),
          actions: [
            IconButton(
                icon: Icon(Icons.upload_file),
                onPressed: () {
                  _saveBankingFormData();
                })
          ],
        ),
        body: Padding(
            padding: EdgeInsets.all(20),
            child: SingleChildScrollView(
                child: Column(
              mainAxisAlignment: MainAxisAlignment.start,
              crossAxisAlignment: CrossAxisAlignment.start,
              children: [
                _getClientForm(),
                SizedBox(height: 25),
                _getDirectCreditForm(_showDirectCredit)
              ],
            ))));
  }

  Widget _getClientForm() {
    return MultiProvider(
        providers: [
          ChangeNotifierProvider(create: (_) => WidgetPropChangeNotifier())
        ],
        child: FormBuilder(
            skipDisabled: true,
            key: clientFormKey,
            initialValue: {
              'clientName': 'Siti Maisarah binti Ahmad Bakri',
              'clientPhone': '0122805469',
              'agentName': 'John Doe'
            },
            autovalidateMode: AutovalidateMode.always,
            child: Card(
                elevation: 8.0,
                color: Colors.white,
                shape: RoundedRectangleBorder(
                    borderRadius: BorderRadius.circular(32)),
                child: Padding(
                    padding: EdgeInsets.all(20),
                    child: Column(
                      crossAxisAlignment: CrossAxisAlignment.start,
                      children: [
                        Row(
                          children: [
                            Column(
                              mainAxisSize: MainAxisSize.min,
                              crossAxisAlignment: CrossAxisAlignment.start,
                              children: [
                                Text('Claim - Client Info',
                                    style: TextStyle(
                                        fontSize: 24,
                                        fontFamily: 'Lato',
                                        fontWeight: FontWeight.bold,
                                        color: Colors.black)),
                                SizedBox(height: 3),
                                Text('Admission to Hospital (Inpatient bill)',
                                    style: TextStyle(
                                        fontSize: 16,
                                        fontFamily: 'Lato',
                                        fontWeight: FontWeight.normal,
                                        color: HexColor.fromHex("#9599B6"))),
                              ],
                            ),
                            Expanded(child: Container()),
                            RichText(
                                text: TextSpan(
                                    text: '0',
                                    style: TextStyle(
                                        fontSize: 24,
                                        fontFamily: 'Lato',
                                        fontWeight: FontWeight.bold,
                                        color: Colors.black),
                                    children: [
                                  TextSpan(
                                      text: ' / 2   Mandatory Items',
                                      style: TextStyle(
                                          fontSize: 16,
                                          fontFamily: 'Lato',
                                          fontWeight: FontWeight.normal,
                                          color: Colors.black))
                                ]))
                          ],
                        ),
                        Container(
                            margin: EdgeInsets.fromLTRB(0, 20, 0, 20),
                            height: 1,
                            width: double.infinity,
                            color: HexColor.fromHex("#9599B6")),
                        FullSizeColumn(
                            id: 'agentNameCol',
                            label: 'Agent Name',
                            visible: true,
                            compulsory: true,
                            child: TxtField(fieldProp: {
                              'inputId': 'agentName',
                              'formId': 'clientForm',
                              'inputType': TxtFieldInputType.NAME.type,
                              'validators': {
                                'nameErrMsg': 'Invalid name format',
                                'compulsory': true,
                                'compulsoryErrMsg': 'This is compulsory.',
                                'currencyErrMsg': 'Invalid currency value.',
                                'maxVal': 0,
                                'maxValErrMsg':
                                    'Input value cannot greater than 3000.',
                                'maxLength': 20,
                                'maxLengthErrMsg':
                                    'Input value length cannot greater than 20',
                              },
                              'enabled': false,
                              'label': '',
                              'hint': 'Full Name'
                            }, listener: DefaultClickListener())),
                        FullSizeColumn(
                            id: 'clientNameCol',
                            label: 'Client Name',
                            visible: true,
                            compulsory: true,
                            child: TxtField(fieldProp: {
                              'inputId': 'clientName',
                              'formId': 'clientForm',
                              'inputType': TxtFieldInputType.NAME.type,
                              'validators': {
                                'nameErrMsg': 'Invalid name format',
                                'compulsory': true,
                                'compulsoryErrMsg': 'This is compulsory.',
                                'currencyErrMsg': 'Invalid currency value.',
                                'maxVal': 0,
                                'maxValErrMsg':
                                    'Input value cannot greater than 3000.',
                                'maxLength': 40,
                                'maxLengthErrMsg':
                                    'Input value length cannot greater than 20',
                              },
                              'enabled': true,
                              'label': '',
                              'hint': 'Full Name'
                            }, listener: DefaultClickListener())),
                        FullSizeColumn(
                            id: 'clientPhoneCol',
                            label: 'Phone Number',
                            visible: true,
                            compulsory: true,
                            child: TxtField(fieldProp: {
                              'inputId': 'clientPhone',
                              'formId': 'clientForm',
                              'inputType': TxtFieldInputType.PHONE.type,
                              'validators': {
                                'phoneErrMsg': 'Invalid phone format',
                                'compulsory': true,
                                'compulsoryErrMsg': 'This is compulsory.',
                                'currencyErrMsg': 'Invalid currency value.',
                                'maxVal': 0,
                                'maxValErrMsg':
                                    'Input value cannot greater than 3000.',
                                'maxLength': 12,
                                'maxLengthErrMsg':
                                    'Input value length cannot greater than 12',
                              },
                              'enabled': true,
                              'label': '',
                              'hint': 'Mobile/Home Phone'
                            }, listener: DefaultClickListener())),
                        FullSizeColumn(
                            id: 'directCreditButRowCol',
                            label: 'Direct Credit Info',
                            visible: true,
                            compulsory: true,
                            child: Row(
                                mainAxisAlignment: MainAxisAlignment.start,
                                children: [
                                  ElevatedButton(
                                      style: ElevatedButton.styleFrom(
                                          primary: HexColor.fromHex(
                                              'ee1c25'), // background
                                          onPrimary: Colors.white, // foreground
                                          textStyle: TextStyle(
                                              fontFamily: 'Lato',
                                              fontWeight: FontWeight.bold)),
                                      onPressed: () {
                                        print('fill up here, button press');
                                        if (!_showDirectCredit) {
                                          setState(() {
                                            _showDirectCredit = true;
                                          });
                                        } else {
                                          animationCtrl?.forward();
                                        }
                                      },
                                      child: Text('Fill up here')),
                                  SizedBox(width: 20),
                                  ElevatedButton(
                                      style: ElevatedButton.styleFrom(
                                          primary: HexColor.fromHex(
                                              'be0e17'), // background
                                          onPrimary: Colors.white,
                                          textStyle: TextStyle(
                                              fontFamily: 'Lato',
                                              fontWeight:
                                                  FontWeight.bold) // foreground
                                          ),
                                      // style: ButtonStyle(
                                      //   backgroundColor: MaterialStateProperty.resolveWith<Color>(
                                      //     (Set<MaterialState> states) {
                                      //       // if (states.contains(MaterialState.pressed))
                                      //       //   return Colors.green;
                                      //       // return null; // Use the component's default.
                                      //     },
                                      //   ),
                                      // ),
                                      onPressed: () {
                                        // setState(() {
                                        //   _showDirectCredit = false;
                                        // });
                                        animationCtrl?.reverse();
                                      },
                                      child: Text('Customer to fill up'))
                                ]))
                      ],
                    )))));
  }

  Widget _getDirectCreditForm(bool show) {
    return BounceInDown(
        animate: show,
        manualTrigger: true,
        controller: (animationController) {
          animationCtrl = animationController;
        },
        child: /*Expanded(
          child:*/
            Container(
                width: double.infinity,
                //width: Util.getWidthPctOfScreen(context, 85),
                child: FormAndLayoutBuild(
                    source: _source,
                    genInitFormData: () {
                      return Map<String, dynamic>();
                    },
                    pageId: _pageId)) //),
        );
  }
}
