import 'package:flutter/material.dart';
import 'package:dynamic_widget/dynamic_widget.dart';

import 'package:flutter/cupertino.dart';
import 'package:flutter_form_builder_ing/flutter_form_builder_ing.dart';
import 'package:layout_form_builder/datalayer/bean/eform_cache.dart';
import 'package:layout_form_builder/ui/app_colors.dart';

import 'dart:async';
import 'package:provider/provider.dart';
import 'datalayer/bean/eform_page_json_form_data.dart';
import 'widget_prop_change_notifier.dart';

class LayoutForm extends StatefulWidget {
  final ClickListener clickListener;
  final PageJsonFormData pageJsonFormData;

  LayoutForm({Key? key, required this.clickListener, required this.pageJsonFormData}) : super(key: key);

  @override
  _LayoutFormState createState() => _LayoutFormState();
}

class _LayoutFormState extends State<LayoutForm> {
  @override
  Widget build(BuildContext context) {
    return MultiProvider(
      providers: [
        ChangeNotifierProvider(create: (_) => WidgetPropChangeNotifier())
      ],
      child: FormBuilder(
        skipDisabled: true,
        key: EFormCache.getFormKey(widget.pageJsonFormData.pageId),
        initialValue: widget.pageJsonFormData.formData,
        autovalidateMode: AutovalidateMode.always,
        child: _LayoutFormRenderer(
          clickListener: widget.clickListener,
          jsonStr: widget.pageJsonFormData.pageJson
        ),
      )
    );
  }
}

class _LayoutFormRenderer extends StatelessWidget {
  final String jsonStr;
  final ClickListener clickListener;

  _LayoutFormRenderer({Key? key, required this.clickListener, required this.jsonStr}) : super(key: key);

  @override
  Widget build(BuildContext context) {
    return FutureBuilder<Widget?>(
        future: _buildWidget(context),
        builder: (BuildContext context, AsyncSnapshot<Widget?> snapshot) {
          if (snapshot.hasError) {
            return Center(
              child: Text('Failed to render the UI due to ${snapshot.error}'));
          }
          return snapshot.hasData
            ? snapshot.data ?? Container()
            : Center(
                child: Column(
                mainAxisSize: MainAxisSize.min,
                children: [
                  SizedBox(width: 30, height: 30, child: CircularProgressIndicator()),
                  SizedBox(height: 5),
                  Text(
                    'Loading ...',
                    style: TextStyle(
                      color: AppColors.GrayLineLight,
                      fontSize: 14,
                      decoration: TextDecoration.none,
                    )
                  )
                ],
              )
          );
        });
  }

  Future<Widget?> _buildWidget(BuildContext context) async {
    try {
      print("[LayoutFormRenderer] Rendering dynamic widget with json : $jsonStr");
      return DynamicWidgetBuilder.build(jsonStr, context, clickListener);
    } catch (e, s) {
      print("[LayoutFormRenderer] Failed to render json preview with error: $e, stacktrace: $s");
      throw e;
    }
  }
}
